@@ -1,5 +1,5 @@
 /**
- * @file libnotify/notifynotification.c Notification object
+ * @file libnotify/notification.c Notification object
  *
  * @Copyright (C) 2006 Christian Hammond
  * @Copyright (C) 2006 John Palmieri
@@ -26,10 +26,23 @@
 #include <libnotify/notify.h>
 #include <libnotify/internal.h>
 
+#include <gtk/gtkversion.h>
+#if GTK_CHECK_VERSION(2, 9, 2)
+# define HAVE_STATUS_ICON
+# include <gtk/gtkstatusicon.h>
+#endif
+
 #define CHECK_DBUS_VERSION(major, minor) \
 	(DBUS_MAJOR_VER > (major) || \
 	 (DBUS_MAJOR_VER == (major) && DBUS_MINOR_VER >= (minor)))
 
+#if !defined(G_PARAM_STATIC_NAME) && !defined(G_PARAM_STATIC_NICK) && \
+    !defined(G_PARAM_STATIC_BLURB)
+# define G_PARAM_STATIC_NAME 0
+# define G_PARAM_STATIC_NICK 0
+# define G_PARAM_STATIC_BLURB 0
+#endif
+
 static void notify_notification_class_init(NotifyNotificationClass *klass);
 static void notify_notification_init(NotifyNotification *sp);
 static void notify_notification_finalize(GObject *object);
@@ -69,8 +82,9 @@ struct _NotifyNotificationPrivate
 	GHashTable *hints;
 
 	GtkWidget *attached_widget;
-	gint widget_old_x;
-	gint widget_old_y;
+#ifdef HAVE_STATUS_ICON
+	GtkStatusIcon *status_icon;
+#endif
 
 	gboolean has_nondefault_actions;
 	gboolean updates_pending;
@@ -83,10 +97,38 @@ enum
 	LAST_SIGNAL
 };
 
+enum
+{
+	PROP_0,
+	PROP_SUMMARY,
+	PROP_BODY,
+	PROP_ICON_NAME,
+	PROP_ATTACH_WIDGET,
+	PROP_STATUS_ICON
+};
+
+static void notify_notification_set_property(GObject *object, guint prop_id,
+											 const GValue *value,
+											 GParamSpec *pspec);
+static void notify_notification_get_property(GObject *object, guint prop_id,
+											 GValue *value, GParamSpec *pspec);
 static guint signals[LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class = NULL;
 
-G_DEFINE_TYPE(NotifyNotification, notify_notification, G_TYPE_OBJECT);
+G_DEFINE_TYPE(NotifyNotification, notify_notification, G_TYPE_OBJECT)
+
+static GObject *
+notify_notification_constructor(GType type,
+								guint n_construct_properties,
+								GObjectConstructParam *construct_params)
+{
+	GObject *object = parent_class->constructor(type, n_construct_properties,
+												construct_params);
+
+	_notify_cache_add_notification(NOTIFY_NOTIFICATION(object));
+
+	return object;
+}
 
 static void
 notify_notification_class_init(NotifyNotificationClass *klass)
@@ -95,9 +137,17 @@ notify_notification_class_init(NotifyNotificationClass *klass)
 
 	parent_class = g_type_class_peek_parent(klass);
 
+	object_class->constructor = notify_notification_constructor;
+	object_class->get_property = notify_notification_get_property;
+	object_class->set_property = notify_notification_set_property;
 	object_class->finalize = notify_notification_finalize;
 
-	/* Create signals here: */
+	/**
+	 * NotifyNotification::closed:
+	 * @notification: The object which received the signal.
+	 *
+	 * Emitted when the notification is closed.
+	 */
 	signals[SIGNAL_CLOSED] =
 		g_signal_new("closed",
 					 G_TYPE_FROM_CLASS(object_class),
@@ -105,6 +155,145 @@ notify_notification_class_init(NotifyNotificationClass *klass)
 					 G_STRUCT_OFFSET(NotifyNotificationClass, closed),
 					 NULL, NULL,
 					 g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
+	g_object_class_install_property(object_class, PROP_SUMMARY,
+		g_param_spec_string("summary", "Summary",
+							"The summary text",
+							NULL,
+							G_PARAM_READWRITE |
+							G_PARAM_CONSTRUCT |
+							G_PARAM_STATIC_NAME |
+							G_PARAM_STATIC_NICK |
+							G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property(object_class, PROP_BODY,
+		g_param_spec_string("body", "Message Body",
+							"The message body text",
+							NULL,
+							G_PARAM_READWRITE |
+							G_PARAM_CONSTRUCT |
+							G_PARAM_STATIC_NAME |
+							G_PARAM_STATIC_NICK |
+							G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property(object_class, PROP_ICON_NAME,
+		g_param_spec_string("icon-name",
+							"Icon Name",
+							"The icon filename or icon theme-compliant name",
+							NULL,
+							G_PARAM_READWRITE |
+							G_PARAM_CONSTRUCT |
+							G_PARAM_STATIC_NAME |
+							G_PARAM_STATIC_NICK |
+							G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property(object_class, PROP_ATTACH_WIDGET,
+		g_param_spec_object("attach-widget",
+							"Attach Widget",
+							"The widget to attach the notification to",
+							GTK_TYPE_WIDGET,
+							G_PARAM_READWRITE |
+							G_PARAM_CONSTRUCT |
+							G_PARAM_STATIC_NAME |
+							G_PARAM_STATIC_NICK |
+							G_PARAM_STATIC_BLURB));
+
+#ifdef HAVE_STATUS_ICON
+	g_object_class_install_property(object_class, PROP_STATUS_ICON,
+		g_param_spec_object("status-icon",
+							"Status Icon",
+							"The status icon to attach the notification to",
+							GTK_TYPE_STATUS_ICON,
+							G_PARAM_READWRITE |
+							G_PARAM_CONSTRUCT |
+							G_PARAM_STATIC_NAME |
+							G_PARAM_STATIC_NICK |
+							G_PARAM_STATIC_BLURB));
+#endif /* HAVE_STATUS_ICON */
+}
+
+static void
+notify_notification_set_property(GObject *object,
+                                 guint prop_id,
+                                 const GValue *value,
+                                 GParamSpec *pspec)
+{
+	NotifyNotification *notification = NOTIFY_NOTIFICATION(object);
+	NotifyNotificationPrivate *priv = notification->priv;
+
+	switch (prop_id)
+	{
+		case PROP_SUMMARY:
+			notify_notification_update(notification, g_value_get_string(value),
+									   priv->body, priv->icon_name);
+			break;
+
+		case PROP_BODY:
+			notify_notification_update(notification, priv->summary,
+									   g_value_get_string(value),
+									   priv->icon_name);
+			break;
+
+		case PROP_ICON_NAME:
+			notify_notification_update(notification, priv->summary,
+									   priv->body, g_value_get_string(value));
+			break;
+
+		case PROP_ATTACH_WIDGET:
+			notify_notification_attach_to_widget(notification,
+				g_value_get_object(value));
+			break;
+
+#ifdef HAVE_STATUS_ICON
+		case PROP_STATUS_ICON:
+			notify_notification_attach_to_status_icon(notification,
+				g_value_get_object(value));
+			break;
+#endif
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+			break;
+	}
+}
+
+static void
+notify_notification_get_property(GObject *object,
+                                 guint prop_id,
+                                 GValue *value,
+                                 GParamSpec *pspec)
+{
+	NotifyNotification *notification = NOTIFY_NOTIFICATION(object);
+	NotifyNotificationPrivate *priv = notification->priv;
+
+	switch (prop_id)
+	{
+		case PROP_SUMMARY:
+			g_value_set_string(value, priv->summary);
+			break;
+
+		case PROP_BODY:
+			g_value_set_string(value, priv->body);
+			break;
+
+		case PROP_ICON_NAME:
+			g_value_set_string(value, priv->icon_name);
+			break;
+
+		case PROP_ATTACH_WIDGET:
+			g_value_set_object(value, priv->attached_widget);
+			break;
+
+#ifdef HAVE_STATUS_ICON
+		case PROP_STATUS_ICON:
+			g_value_set_object(value, priv->status_icon);
+			break;
+#endif
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+			break;
+	}
 }
 
 static void
@@ -165,6 +354,12 @@ notify_notification_finalize(GObject *object)
 	if (priv->attached_widget != NULL)
 		g_object_unref(G_OBJECT(priv->attached_widget));
 
+#ifdef HAVE_STATUS_ICON
+    if (priv->status_icon != NULL)
+        g_object_remove_weak_pointer(G_OBJECT(priv->status_icon),
+								     (gpointer)&priv->status_icon);
+#endif
+
 	if (priv->signals_registered)
 	{
 		dbus_g_proxy_disconnect_signal(proxy, "NotificationClosed",
@@ -180,21 +375,34 @@ notify_notification_finalize(GObject *object)
 	G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
-static gboolean
+static void
 _notify_notification_update_applet_hints(NotifyNotification *n)
 {
 	NotifyNotificationPrivate *priv = n->priv;
-	gboolean update_pending = FALSE;
+	GdkScreen *screen = NULL;
+	gint x, y;
 
+#ifdef HAVE_STATUS_ICON
+	if (priv->status_icon != NULL)
+	{
+		GdkRectangle rect;
+
+		if (!gtk_status_icon_get_geometry(priv->status_icon, &screen,
+										  &rect, NULL))
+		{
+			return;
+		}
+
+		x = rect.x + rect.width / 2;
+		y = rect.y + rect.height / 2;
+	}
+	else
+#endif /* HAVE_STATUS_ICON */
 	if (priv->attached_widget != NULL)
 	{
-		gint x, y, h, w;
 		GtkWidget *widget = priv->attached_widget;
-		GtkRequisition requisition;
 
-		gtk_widget_size_request(widget, &requisition);
-		w = requisition.width;
-		h = requisition.height;
+		screen = gtk_widget_get_screen(widget);
 
 		gdk_window_get_origin(widget->window, &x, &y);
 
@@ -206,35 +414,21 @@ _notify_notification_update_applet_hints(NotifyNotification *n)
 
 		x += widget->allocation.width / 2;
 		y += widget->allocation.height / 2;
-
-		if (x != priv->widget_old_x)
-		{
-			notify_notification_set_hint_int32(n, "x", x);
-			priv->widget_old_x = x;
-
-			update_pending = TRUE;
-		}
-
-		if (y != priv->widget_old_y)
-		{
-			notify_notification_set_hint_int32(n, "y", y);
-			priv->widget_old_y = y;
-
-			update_pending = TRUE;
-		}
 	}
+	else
+		return;
 
-	return update_pending;
+	notify_notification_set_geometry_hints(n, screen, x, y);
 }
 
 #if 0
-
-/* This is left here just incase we revisit autoupdating
-   One thought would be to check for updates every time the icon
-   is redrawn but we still have to deal with the race conditions
-   that could occure between the server and the client so we will
-   leave this alone for now.
-*/
+/*
+ * This is left here just incase we revisit autoupdating
+ * One thought would be to check for updates every time the icon
+ * is redrawn but we still have to deal with the race conditions
+ * that could occure between the server and the client so we will
+ * leave this alone for now.
+ */
 static gboolean
 _idle_check_updates(void *user_data)
 {
@@ -260,36 +454,79 @@ _idle_check_updates(void *user_data)
 }
 #endif
 
+/**
+ * notify_notification_new:
+ * @summary: The required summary text.
+ * @body: The optional body text.
+ * @icon: The optional icon theme icon name or filename.
+ * @attach: The optional widget to attach to.
+ *
+ * Creates a new #NotifyNotification. The summary text is required, but
+ * all other parameters are optional.
+ *
+ * Returns: The new #NotifyNotification.
+ */
 NotifyNotification *
 notify_notification_new(const gchar *summary, const gchar *body,
 						const gchar *icon, GtkWidget *attach)
 {
-	NotifyNotification *obj;
-
-	g_return_val_if_fail(summary != NULL && *summary != '\0',     NULL);
 	g_return_val_if_fail(attach == NULL || GTK_IS_WIDGET(attach), NULL);
 
-	obj = NOTIFY_NOTIFICATION(g_object_new(NOTIFY_TYPE_NOTIFICATION, NULL));
-
-	obj->priv->summary = g_strdup(summary);
-
-	if (body != NULL && *body != '\0')
-		obj->priv->body = g_strdup(body);
-
-	if (icon != NULL && *icon != '\0')
-		obj->priv->icon_name = g_strdup(icon);
-
-	if (attach != NULL)
-	{
-		g_object_ref(G_OBJECT(attach));
-		obj->priv->attached_widget = attach;
-	}
-
-	_notify_cache_add_notification(obj);
-
-	return obj;
+	return g_object_new(NOTIFY_TYPE_NOTIFICATION,
+						"summary", summary,
+						"body", body,
+						"icon-name",  icon,
+						"attach-widget", attach,
+						NULL);
 }
 
+#ifdef HAVE_STATUS_ICON
+/**
+ * notify_notification_new_with_status_icon:
+ * @summary: The required summary text.
+ * @body: The optional body text.
+ * @icon: The optional icon theme icon name or filename.
+ * @status_icon: The required #GtkStatusIcon.
+ *
+ * Creates a new #NotifyNotification and attaches to a #GtkStatusIcon.
+ * The summary text and @status_icon is required, but all other parameters
+ * are optional.
+ *
+ * Returns: The new #NotifyNotification.
+ *
+ * Since: 0.4.1
+ */
+NotifyNotification *
+notify_notification_new_with_status_icon(const gchar *summary,
+                                         const gchar *message,
+                                         const gchar *icon,
+                                         GtkStatusIcon *status_icon)
+{
+	g_return_val_if_fail(status_icon == NULL, NULL);
+	g_return_val_if_fail(GTK_IS_STATUS_ICON(status_icon), NULL);
+
+	return g_object_new(NOTIFY_TYPE_NOTIFICATION,
+						"summary", summary,
+						"body", message,
+						"icon-name",  icon,
+						"status-icon", status_icon,
+						NULL);
+}
+#endif /* HAVE_STATUS_ICON */
+
+/**
+ * notify_notification_update:
+ * @notification: The notification to update.
+ * @summary: The new required summary text.
+ * @body: The optional body text.
+ * @icon: The optional icon theme icon name or filename.
+ *
+ * Updates the notification text and icon. This won't send the update out
+ * and display it on the screen. For that, you will need to call
+ * notify_notification_show().
+ *
+ * Returns: %TRUE, unless an invalid parameter was passed.
+ */
 gboolean
 notify_notification_update(NotifyNotification *notification,
 						   const gchar *summary, const gchar *body,
@@ -299,35 +536,137 @@ notify_notification_update(NotifyNotification *notification,
 	g_return_val_if_fail(NOTIFY_IS_NOTIFICATION(notification), FALSE);
 	g_return_val_if_fail(summary != NULL && *summary != '\0',  FALSE);
 
-	g_free(notification->priv->summary);
-	g_free(notification->priv->body);
-	g_free(notification->priv->icon_name);
+	if (notification->priv->summary != summary)
+	{
+		g_free(notification->priv->summary);
+		notification->priv->summary = g_strdup(summary);
+		g_object_notify(G_OBJECT(notification), "summary");
+	}
 
-	notification->priv->summary = g_strdup(summary);
+	if (notification->priv->body != body)
+	{
+		g_free(notification->priv->body);
+		notification->priv->body =
+			(body != NULL && *body != '\0' ? g_strdup(body) : NULL);
+		g_object_notify(G_OBJECT(notification), "body");
+	}
 
-	if (body != NULL && *body != '\0')
-		notification->priv->body = g_strdup(body);
-
-	if (icon != NULL && *icon != '\0')
-		notification->priv->icon_name = g_strdup(icon);
+	if (notification->priv->icon_name != icon)
+	{
+		g_free(notification->priv->icon_name);
+		notification->priv->icon_name =
+			(icon != NULL && *icon != '\0' ? g_strdup(icon) : NULL);
+		g_object_notify(G_OBJECT(notification), "icon-name");
+	}
 
 	notification->priv->updates_pending = TRUE;
 
 	return TRUE;
 }
 
+/**
+ * notify_notification_attach_to_widget:
+ * @notification: The notification.
+ * @attach: The widget to attach to, or %NULL.
+ *
+ * Attaches the notification to a widget. This will set hints on the
+ * notification requesting that the notification point to the widget's
+ * location. If @attach is %NULL, the widget will be unset.
+ */
 void
 notify_notification_attach_to_widget(NotifyNotification *notification,
 									 GtkWidget *attach)
 {
-	g_return_if_fail(notification != NULL);
 	g_return_if_fail(NOTIFY_IS_NOTIFICATION(notification));
 
+	if (notification->priv->attached_widget == attach)
+		return;
+
 	if (notification->priv->attached_widget != NULL)
 		g_object_unref(notification->priv->attached_widget);
 
 	notification->priv->attached_widget =
 		(attach != NULL ? g_object_ref(attach) : NULL);
+
+	g_object_notify(G_OBJECT(notification), "attach-widget");
+}
+
+#ifdef HAVE_STATUS_ICON
+/**
+ * notify_notification_attach_to_status_icon:
+ * @notification: The notification.
+ * @status_icon: The #GtkStatusIcon to attach to, or %NULL.
+ *
+ * Attaches the notification to a #GtkStatusIcon. This will set hints on the
+ * notification requesting that the notification point to the status icon's
+ * location. If @status_icon is %NULL, the status icon will be unset.
+ *
+ * Since: 0.4.1
+ */
+void
+notify_notification_attach_to_status_icon(NotifyNotification *notification,
+										  GtkStatusIcon *status_icon)
+{
+	NotifyNotificationPrivate *priv;
+
+	g_return_if_fail(NOTIFY_IS_NOTIFICATION(notification));
+	g_return_if_fail(status_icon == NULL || GTK_IS_STATUS_ICON(status_icon));
+
+	priv = notification->priv;
+
+	if (priv->status_icon == status_icon)
+		return;
+
+	if (priv->status_icon != NULL)
+	{
+		g_object_remove_weak_pointer(G_OBJECT(priv->status_icon),
+									 (gpointer)&priv->status_icon);
+	}
+
+	priv->status_icon = status_icon;
+
+	if (priv->status_icon != NULL)
+	{
+		g_object_add_weak_pointer(G_OBJECT(priv->status_icon),
+								  (gpointer)&priv->status_icon);
+	}
+
+	g_object_notify(G_OBJECT(notification), "attach-icon");
+}
+#endif /* HAVE_STATUS_ICON */
+
+/**
+ * notify_notification_set_geometry_hints:
+ * @notification: The notification.
+ * @screen: The #GdkScreen the notification should appear on.
+ * @x: The X coordinate to point to.
+ * @y: The Y coordinate to point to.
+ *
+ * Sets the geometry hints on the notification. This sets the screen
+ * the notification should appear on and the X, Y coordinates it should
+ * point to, if the particular notification supports X, Y hints.
+ *
+ * Since: 0.4.1
+ */
+void
+notify_notification_set_geometry_hints(NotifyNotification *notification,
+									   GdkScreen *screen,
+									   gint x,
+									   gint y)
+{
+	char *display_name;
+
+	g_return_if_fail(notification != NULL);
+	g_return_if_fail(NOTIFY_IS_NOTIFICATION(notification));
+	g_return_if_fail(screen != NULL);
+	g_return_if_fail(GDK_IS_SCREEN(screen));
+
+	notify_notification_set_hint_int32(notification, "x", x);
+	notify_notification_set_hint_int32(notification, "y", y);
+
+	display_name = gdk_screen_make_display_name(screen);
+	notify_notification_set_hint_string(notification, "xdisplay", display_name);
+	g_free(display_name);
 }
 
 static void
@@ -354,9 +693,14 @@ _action_signal_handler(DBusGProxy *proxy, guint32 id, gchar *action,
 		notification->priv->action_map, action);
 
 	if (pair == NULL)
-		g_warning("Recieved unknown action %s", action);
+	{
+		if (g_ascii_strcasecmp(action, "default"))
+			g_warning("Received unknown action %s", action);
+	}
 	else
+	{
 		pair->cb(notification, action, pair->user_data);
+	}
 }
 
 static gchar **
@@ -372,6 +716,16 @@ _gslist_to_string_array(GSList *list)
 	return (gchar **)g_array_free(a, FALSE);
 }
 
+/**
+ * notify_notification_show:
+ * @notification: The notification.
+ * @error: The returned error information.
+ *
+ * Tells the notification server to display the notification on the screen.
+ *
+ * Returns: %TRUE if successful. On error, this will return %FALSE and set
+ *          @error.
+ */
 gboolean
 notify_notification_show(NotifyNotification *notification, GError **error)
 {
@@ -400,7 +754,7 @@ notify_notification_show(NotifyNotification *notification, GError **error)
 		priv->signals_registered = TRUE;
 	}
 
-	/* If attached to a widget, modify x and y in hints */
+	/* If attached to a widget or status icon, modify x and y in hints */
 	_notify_notification_update_applet_hints(notification);
 
 	action_array = _gslist_to_string_array(priv->actions);
@@ -432,6 +786,15 @@ notify_notification_show(NotifyNotification *notification, GError **error)
 	return TRUE;
 }
 
+/**
+ * notify_notification_set_timeout:
+ * @notification: The notification.
+ * @timeout: The timeout in milliseconds.
+ *
+ * Sets the timeout of the notification. To set the default time, pass
+ * %NOTIFY_EXPIRES_DEFAULT as @timeout. To set the notification to never
+ * expire, pass %NOTIFY_EXPIRES_NEVER.
+ */
 void
 notify_notification_set_timeout(NotifyNotification *notification,
 								gint timeout)
@@ -451,6 +814,14 @@ _notify_notification_get_timeout(const NotifyNotification *notification)
 	return notification->priv->timeout;
 }
 
+/**
+ * notify_notification_set_category:
+ * @notification: The notification.
+ * @category: The category.
+ *
+ * Sets the category of this notification. This can be used by the
+ * notification server to filter or display the data in a certain way.
+ */
 void
 notify_notification_set_category(NotifyNotification *notification,
 								 const char *category)
@@ -461,14 +832,23 @@ notify_notification_set_category(NotifyNotification *notification,
 	notify_notification_set_hint_string(notification, "category", category);
 }
 
+/**
+ * notify_notification_set_urgency:
+ * @notification: The notification.
+ * @urgency: The urgency level.
+ *
+ * Sets the urgency level of this notification.
+ *
+ * See: #NotifyUrgency
+ */
 void
 notify_notification_set_urgency(NotifyNotification *notification,
-								NotifyUrgency l)
+								NotifyUrgency urgency)
 {
 	g_return_if_fail(notification != NULL);
 	g_return_if_fail(NOTIFY_IS_NOTIFICATION(notification));
 
-	notify_notification_set_hint_byte(notification, "urgency", (guchar)l);
+	notify_notification_set_hint_byte(notification, "urgency", (guchar)urgency);
 }
 
 #if CHECK_DBUS_VERSION(0, 60)
@@ -529,6 +909,16 @@ _gvalue_array_append_byte_array(GValueArray *array, guchar *bytes, gsize len)
 }
 #endif /* D-BUS >= 0.60 */
 
+/**
+ * notify_notification_set_icon_from_pixbuf:
+ * @notification: The notification.
+ * @icon: The icon.
+ *
+ * Sets the icon in the notification from a #GdkPixbuf.
+ *
+ * This will only work when libnotify is compiled against D-BUS 0.60 or
+ * higher.
+ */
 void
 notify_notification_set_icon_from_pixbuf(NotifyNotification *notification,
 										 GdkPixbuf *icon)
@@ -580,6 +970,14 @@ notify_notification_set_icon_from_pixbuf(NotifyNotification *notification,
 #endif
 }
 
+/**
+ * notify_notification_set_hint_int32:
+ * @notification: The notification.
+ * @key: The hint.
+ * @value: The hint's value.
+ *
+ * Sets a hint with a 32-bit integer value.
+ */
 void
 notify_notification_set_hint_int32(NotifyNotification *notification,
 								   const gchar *key, gint value)
@@ -597,6 +995,14 @@ notify_notification_set_hint_int32(NotifyNotification *notification,
 						g_strdup(key), hint_value);
 }
 
+/**
+ * notify_notification_set_hint_double:
+ * @notification: The notification.
+ * @key: The hint.
+ * @value: The hint's value.
+ *
+ * Sets a hint with a double value.
+ */
 void
 notify_notification_set_hint_double(NotifyNotification *notification,
 									const gchar *key, gdouble value)
@@ -614,6 +1020,14 @@ notify_notification_set_hint_double(NotifyNotification *notification,
 						g_strdup(key), hint_value);
 }
 
+/**
+ * notify_notification_set_hint_byte:
+ * @notification: The notification.
+ * @key: The hint.
+ * @value: The hint's value.
+ *
+ * Sets a hint with a byte value.
+ */
 void
 notify_notification_set_hint_byte(NotifyNotification *notification,
 								  const gchar *key, guchar value)
@@ -631,6 +1045,16 @@ notify_notification_set_hint_byte(NotifyNotification *notification,
 	g_hash_table_insert(notification->priv->hints, g_strdup(key), hint_value);
 }
 
+/**
+ * notify_notification_set_hint_byte_array:
+ * @notification: The notification.
+ * @key: The hint.
+ * @value: The hint's value.
+ * @len: The length of the byte array.
+ *
+ * Sets a hint with a byte array value. The length of @value must be passed
+ * as @len.
+ */
 void
 notify_notification_set_hint_byte_array(NotifyNotification *notification,
 										const gchar *key,
@@ -657,6 +1081,14 @@ notify_notification_set_hint_byte_array(NotifyNotification *notification,
 						g_strdup(key), hint_value);
 }
 
+/**
+ * notify_notification_set_hint_string:
+ * @notification: The notification.
+ * @key: The hint.
+ * @value: The hint's value.
+ *
+ * Sets a hint with a string value.
+ */
 void
 notify_notification_set_hint_string(NotifyNotification *notification,
 									const gchar *key, const gchar *value)
@@ -680,6 +1112,12 @@ _remove_all(void)
 	return TRUE;
 }
 
+/**
+ * notify_notification_clear_hints:
+ * @notification: The notification.
+ *
+ * Clears all hints from the notification.
+ */
 void
 notify_notification_clear_hints(NotifyNotification *notification)
 {
@@ -690,6 +1128,12 @@ notify_notification_clear_hints(NotifyNotification *notification)
 								(GHRFunc)_remove_all, NULL);
 }
 
+/**
+ * notify_notification_clear_actions:
+ * @notification: The notification.
+ *
+ * Clears all actions from the notification.
+ */
 void
 notify_notification_clear_actions(NotifyNotification *notification)
 {
@@ -709,6 +1153,20 @@ notify_notification_clear_actions(NotifyNotification *notification)
 	notification->priv->has_nondefault_actions = FALSE;
 }
 
+/**
+ * notify_notification_add_action:
+ * @notification: The notification.
+ * @action: The action ID.
+ * @label: The human-readable action label.
+ * @callback: The action's callback function.
+ * @user_data: Optional custom data to pass to @callback.
+ * @free_func: An optional function to free @user_data when the notification
+ *             is destroyed.
+ *
+ * Adds an action to a notification. When the action is invoked, the
+ * specified callback function will be called, along with the value passed
+ * to @user_data.
+ */
 void
 notify_notification_add_action(NotifyNotification *notification,
 							   const char *action,
@@ -751,6 +1209,16 @@ _notify_notification_has_nondefault_actions(const NotifyNotification *n)
 	return n->priv->has_nondefault_actions;
 }
 
+/**
+ * notify_notification_close:
+ * @notification: The notification.
+ * @error: The returned error information.
+ *
+ * Tells the notification server to hide the notification on the screen.
+ *
+ * Returns: %TRUE if successful. On error, this will return %FALSE and set
+ *          @error.
+ */
 gboolean
 notify_notification_close(NotifyNotification *notification,
 						  GError **error)
